@@ -1,6 +1,6 @@
 /* horst - Highly Optimized Radio Scanning Tool
  *
- * Copyright (C) 2005-2014 Bruno Randolf (br1@einfach.org)
+ * Copyright (C) 2005-2016 Bruno Randolf (br1@einfach.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,16 +22,12 @@
 
 #include <stddef.h>
 
-int
-open_packet_socket(char* devname, int recv_buffer_size);
+int open_packet_socket(char* devname, int recv_buffer_size);
 
-int
-recv_packet(int fd, unsigned char* buffer, size_t bufsize);
+int recv_packet(int fd, unsigned char* buffer, size_t bufsize);
 
-void
-close_packet_socket(int fd, char* ifname);
+void close_packet_socket(int fd);
 
-int
-device_get_hwinfo(int fd, char* ifname, unsigned char* mac);
+int device_get_hwinfo(int fd, char* ifname, unsigned char* mac);
 
 #endif // _CAPTURE_H_
