@@ -1,6 +1,6 @@
 /* horst - Highly Optimized Radio Scanning Tool
  *
- * Copyright (C) 2005-2014 Bruno Randolf (br1@einfach.org)
+ * Copyright (C) 2005-2016 Bruno Randolf (br1@einfach.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,18 +20,13 @@
 #ifndef _CONTROL_H_
 #define _CONTROL_H_
 
+#define DEFAULT_CONTROL_PIPE "/tmp/horst"
+
 extern int ctlpipe;
 
-void
-control_init_pipe();
-
-void
-control_send_command(char* cmd);
-
-void
-control_receive_command();
-
-void
-control_finish(void);
+void control_init_pipe();
+void control_send_command(const char* cmd);
+void control_receive_command();
+void control_finish(void);
 
 #endif
