@@ -1,6 +1,6 @@
 /* horst - Highly Optimized Radio Scanning Tool
  *
- * Copyright (C) 2005-2014 Bruno Randolf (br1@einfach.org)
+ * Copyright (C) 2005-2016 Bruno Randolf (br1@einfach.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,12 +25,9 @@
 #include "main.h"
 #include "wlan_util.h"
 
-
-void
-update_help_win(WINDOW *win)
+void update_help_win(WINDOW *win)
 {
-	int i, l;
-	struct pkt_name c;
+	int i, t, col, l;
 
 	werase(win);
 	wattron(win, WHITE);
@@ -40,35 +37,39 @@ update_help_win(WINDOW *win)
 	print_centered(win, 3, COLS, "HORST - Highly Optimized Radio Scanning Tool");
 
 	print_centered(win, 5, COLS, "Version " VERSION " (build date " __DATE__ " " __TIME__ ")");
-	print_centered(win, 6, COLS, "(C) 2005-2014 Bruno Randolf, Licensed under the GPLv2");
+	print_centered(win, 6, COLS, "(C) 2005-2016 Bruno Randolf, Licensed under the GPLv2");
 
 	mvwprintw(win, 8, 2, "Known IEEE802.11 Packet Types:");
 
-	l = 10;
-	/* this is weird but it works */
-	mvwprintw(win, l++, 2, "MANAGEMENT FRAMES");
-	for (i = 0x00; i <= 0xE0; i = i + 0x10) {
-		c = get_packet_struct(i);
-		if (c.c != '?')
-			mvwprintw(win, l++, 4, "%c  %-6s  %s", c.c, c.name, c.desc);
+	for (t = 0; t < WLAN_NUM_TYPES; t++) {
+		wattron(win, A_BOLD);
+		if (t == 0) {
+			l = 10;
+			col = 4;
+			mvwprintw(win, l++, 2, "MANAGEMENT FRAMES");
+		}
+		else if (t == 1) {
+			l++;
+			mvwprintw(win, l++, 2, "CONTROL FRAMES");
+
+		} else {
+			l = 10;
+			col = 47;
+			mvwprintw(win, l++, 45, "DATA FRAMES");
+		}
+		wattroff(win, A_BOLD);
+
+		for (i = 0; i < WLAN_NUM_STYPES; i++) {
+			if (stype_names[t][i].c != '-')
+				mvwprintw(win, l++, col, "%c  %-6s  %s",
+					  stype_names[t][i].c,
+					  stype_names[t][i].name,
+					  stype_names[t][i].desc);
+		}
 	}
 
-	l = 10;
-	mvwprintw(win, l++, 45, "DATA FRAMES");
-	for (i = 0x08; i <= 0xF8; i = i + 0x10) {
-		c = get_packet_struct(i);
-		if (c.c != '?')
-			mvwprintw(win, l++, 47, "%c  %-6s  %s", c.c, c.name, c.desc);
-	}
-
-	mvwprintw(win, l++, 2, "CONTROL FRAMES");
-	for (i = 0x74; i <= 0xF4; i = i + 0x10) {
-		c = get_packet_struct(i);
-		if (c.c != '?')
-			mvwprintw(win, l++, 4, "%c  %-6s  %s", c.c, c.name, c.desc);
-	}
-
-	print_centered(win, ++l, COLS, "For more info read the man page or check http://br1.einfach.org/horst/");
+	wattron(win, WHITE);
+	print_centered(win, 39, COLS, "For more info read the man page or check http://br1.einfach.org/horst/");
 
 	wrefresh(win);
 }
