@@ -1,6 +1,6 @@
 /* horst - Highly Optimized Radio Scanning Tool
  *
- * Copyright (C) 2005-2014 Bruno Randolf (br1@einfach.org)
+ * Copyright (C) 2005-2016 Bruno Randolf (br1@einfach.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,66 +26,41 @@
 #include <signal.h>
 #include <sys/ioctl.h>
 
+#include "util.h"
 #include "display.h"
 #include "main.h"
 #include "wlan80211.h"
 #include "channel.h"
 
-
 static WINDOW *conf_win = NULL;
 static WINDOW *show_win = NULL;
 static int conf_win_current;
 static int show_win_current;
 
-static struct timeval last_time;
+static struct timespec last_time;
 
 static int display_resize_needed = 0;
 
-/* main windows are special */
-void init_display_main(void);
-void clear_display_main(void);
-void update_main_win(struct packet_info *pkt);
-void update_dump_win(struct packet_info* pkt);
-int main_input(int c);
-void print_dump_win(const char *str, int refresh);
-void resize_display_main(void);
-static void resize_display_all(void);
-
-/* smaller config windows */
-void update_filter_win(WINDOW *win);
-void update_channel_win(WINDOW *win);
-int filter_input(WINDOW *win, int c);
-int channel_input(WINDOW *win, int c);
-
-/* "standard" windows */
-void update_spectrum_win(WINDOW *win);
-void update_statistics_win(WINDOW *win);
-void update_essid_win(WINDOW *win);
-void update_history_win(WINDOW *win);
-void update_help_win(WINDOW *win);
-int spectrum_input(WINDOW *win, int c);
-
 /******************* HELPERS *******************/
 
-void
-get_per_second(unsigned long bytes, unsigned long duration,
+void get_per_second(unsigned long bytes, unsigned long duration,
 	       unsigned long packets, unsigned long retries,
 	       int *bps, int *dps, int *pps, int *rps)
 {
-	static struct timeval last;
+	static struct timespec last;
 	static unsigned long last_bytes, last_dur, last_pkts, last_retr;
 	static int last_bps, last_dps, last_pps, last_rps;
 	float timediff;
 
 	/* reacalculate only every second or more */
-	timediff = (the_time.tv_sec + the_time.tv_usec/1000000.0) -
-		   (last.tv_sec + last.tv_usec/1000000.0);
+	timediff = (time_mono.tv_sec + time_mono.tv_nsec/1000000000.0) -
+		   (last.tv_sec + last.tv_nsec/1000000000.0);
 	if (timediff >= 1.0) {
 		last_dps = (1.0*(duration - last_dur)) / timediff;
 		last_bps = (1.0*(bytes - last_bytes)) / timediff;
 		last_pps = (1.0*(packets - last_pkts)) / timediff;
 		last_rps = (1.0*(retries - last_retr)) / timediff;
-		last = the_time;
+		last = time_mono;
 		last_dur = duration;
 		last_bytes = bytes;
 		last_pkts = packets;
@@ -97,7 +72,6 @@ get_per_second(unsigned long bytes, unsigned long duration,
 	*rps = last_rps;
 }
 
-
 void __attribute__ ((format (printf, 4, 5)))
 print_centered(WINDOW* win, int line, int cols, const char *fmt, ...)
 {
@@ -116,13 +90,11 @@ print_centered(WINDOW* win, int line, int cols, const char *fmt, ...)
 	free(buf);
 }
 
-
-int
-get_packet_type_color(int type)
+int get_packet_type_color(int type)
 {
 	if (type == 1) /* special case for bad FCS */
 		return RED;
-	switch (type & WLAN_FRAME_FC_TYPE_MASK) {
+	switch (WLAN_FRAME_TYPE(type)) {
 		case WLAN_FRAME_TYPE_DATA: return BLUE;
 		case WLAN_FRAME_TYPE_CTRL: return WHITE;
 		case WLAN_FRAME_TYPE_MGMT: return CYAN;
@@ -130,10 +102,7 @@ get_packet_type_color(int type)
 	return YELLOW;
 }
 
-
-void
-signal_average_bar(WINDOW *win, int val, int avg, int y, int x, int height,
-		   int width)
+void signal_average_bar(WINDOW *win, int val, int avg, int y, int x, int height, int width)
 {
 	int i;
 	if (avg <= val) {
@@ -157,10 +126,8 @@ signal_average_bar(WINDOW *win, int val, int avg, int y, int x, int height,
 	wattroff(win, ALLGREEN);
 }
 
-
-void
-general_average_bar(WINDOW *win, int val, int avg, int y, int x,
-		    int width, short color, short color_avg)
+void general_average_bar(WINDOW *win, int val, int avg, int y, int x,
+			 int width, short color, short color_avg)
 {
 	int i;
 	if (avg >= val) {
@@ -184,197 +151,9 @@ general_average_bar(WINDOW *win, int val, int avg, int y, int x,
 	wattroff(win, color_avg);
 }
 
-
-/******************* STATUS *******************/
-
-static void
-update_clock(time_t* sec)
-{
-	static char buf[9];
-	strftime(buf, 9, "%H:%M:%S", localtime(sec));
-	wattron(stdscr, BLACKONWHITE);
-	mvwprintw(stdscr, LINES-1, COLS-9, "|%s", buf);
-	wattroff(stdscr, BLACKONWHITE);
-	wnoutrefresh(stdscr);
-}
-
-
-static void
-update_mini_status(void)
-{
-	wattron(stdscr, BLACKONWHITE);
-	mvwprintw(stdscr, LINES-1, COLS-24, conf.paused ? "|=" : "|>");
-	if (!conf.filter_off && (conf.do_macfilter || conf.filter_pkt != PKT_TYPE_ALL || conf.filter_mode != WLAN_MODE_ALL))
-		mvwprintw(stdscr, LINES-1, COLS-22, "|F");
-	else
-		mvwprintw(stdscr, LINES-1, COLS-22, "| ");
-	mvwprintw(stdscr, LINES-1, COLS-20, "|Ch%02d", channel_get_current_chan());
-	wattroff(stdscr, BLACKONWHITE);
-	wnoutrefresh(stdscr);
-}
-
-
-static void
-update_menu(void)
-{
-	wattron(stdscr, BLACKONWHITE);
-	mvwhline(stdscr, LINES-1, 0, ' ', COLS);
-
-#define KEYMARK A_UNDERLINE
-	attron(KEYMARK); printw("Q"); attroff(KEYMARK); printw("uit ");
-	attron(KEYMARK); printw("P"); attroff(KEYMARK); printw("ause ");
-	attron(KEYMARK); printw("R"); attroff(KEYMARK); printw("eset ");
-	attron(KEYMARK); printw("H"); attroff(KEYMARK); printw("ist ");
-	attron(KEYMARK); printw("E"); attroff(KEYMARK); printw("SSID St");
-	attron(KEYMARK); printw("a"); attroff(KEYMARK); printw("ts ");
-	attron(KEYMARK); printw("S"); attroff(KEYMARK); printw("pec ");
-	attron(KEYMARK); printw("F"); attroff(KEYMARK); printw("ilt ");
-	attron(KEYMARK); printw("C"); attroff(KEYMARK); printw("han ");
-	attron(KEYMARK); printw("?"); attroff(KEYMARK); printw(" ");
-	if (show_win == NULL) {
-		printw("s"); attron(KEYMARK); printw("O"); attroff(KEYMARK); printw("rt");
-	}
-	if (show_win != NULL && show_win_current == 's') {
-		attron(KEYMARK); printw("N"); attroff(KEYMARK); printw("odes");
-	}
-#undef KEYMARK
-	mvwprintw(stdscr, LINES-1, COLS-15, "|%s",
-		  conf.serveraddr ? conf.serveraddr : conf.ifname);
-	wattroff(stdscr, BLACKONWHITE);
-
-	update_mini_status();
-	update_clock(&the_time.tv_sec);
-}
-
-
-/******************* WINDOW MANAGEMENT / UPDATE *******************/
-
-static void
-update_show_win(void)
-{
-	if (show_win_current == 'e')
-		update_essid_win(show_win);
-	else if (show_win_current == 'h')
-		update_history_win(show_win);
-	else if (show_win_current == 'a')
-		update_statistics_win(show_win);
-	else if (show_win_current == 's')
-		update_spectrum_win(show_win);
-	else if (show_win_current == '?')
-		update_help_win(show_win);
-}
-
-
-static void
-show_window(int which)
-{
-	if (show_win != NULL && show_win_current == which) {
-		delwin(show_win);
-		show_win = NULL;
-		show_win_current = 0;
-		update_menu();
-		return;
-	}
-	if (show_win == NULL) {
-		show_win = newwin(LINES-1, COLS, 0, 0);
-		scrollok(show_win, FALSE);
-	}
-	show_win_current = which;
-	update_show_win();
-	update_menu();
-}
-
-
-static void
-show_conf_window(int key)
-{
-	if (conf_win != NULL &&
-	    (conf_win_current == key || key == '\r' || key == KEY_ENTER)) {
-		delwin(conf_win);
-		conf_win = NULL;
-		conf_win_current = 0;
-		return;
-	}
-	if (conf_win == NULL) {
-		if (key == 'f') {
-			conf_win = newwin(27, 57, LINES/2-13, COLS/2-28);
-			update_filter_win(conf_win);
-		}
-		else if (key == 'c') {
-			conf_win = newwin(9, 39, LINES/2-6, COLS/2-20);
-			update_channel_win(conf_win);
-		}
-		scrollok(conf_win, FALSE);
-		conf_win_current = key;
-	}
-}
-
-
-void
-update_display_clock(void)
-{
-	/* helper to update just the clock every second */
-	if (the_time.tv_sec > last_time.tv_sec) {
-		update_clock(&the_time.tv_sec);
-		doupdate();
-	}
-}
-
-
-void
-display_log(const char *string)
-{
-	print_dump_win(string, show_win == NULL);
-}
-
-
-void
-update_display(struct packet_info* pkt)
-{
-	/*
-	 * update only in specific intervals to save CPU time
-	 * if pkt is NULL we want to force an update
-	 */
-	if (pkt != NULL &&
-	    the_time.tv_sec == last_time.tv_sec &&
-	    (the_time.tv_usec - last_time.tv_usec) < conf.display_interval ) {
-		/* just add the line to dump win so we don't loose it */
-		update_dump_win(pkt);
-		return;
-	}
-
-	if (display_resize_needed == 1) {
-		resize_display_all();
-		display_resize_needed = 0;
-	}
-
-	update_menu();
-
-	/* update clock every second */
-	if (the_time.tv_sec > last_time.tv_sec)
-		update_clock(&the_time.tv_sec);
-
-	last_time = the_time;
-
-	if (show_win != NULL)
-		update_show_win();
-	else
-		update_main_win(pkt);
-
-	if (conf_win != NULL) {
-		redrawwin(conf_win);
-		wnoutrefresh(conf_win);
-	}
-
-	/* only one redraw */
-	doupdate();
-}
-
-
 /******************* RESIZE *******************/
 
-static void
-resize_display_all(void)
+static void resize_display_all(void)
 {
 	struct winsize winsz;
 
@@ -399,17 +178,188 @@ resize_display_all(void)
 	}
 }
 
-
-static void
-window_change_handler(__attribute__((unused)) int sig) {
+static void window_change_handler(__attribute__((unused)) int sig) {
 	display_resize_needed = 1;
 }
 
+/******************* STATUS *******************/
+
+static void update_clock(time_t* sec)
+{
+	static char buf[9];
+	strftime(buf, 9, "%H:%M:%S", localtime(sec));
+	wattron(stdscr, BLACKONWHITE);
+	mvwprintw(stdscr, LINES-1, COLS-9, "|%s", buf);
+	wattroff(stdscr, BLACKONWHITE);
+	wnoutrefresh(stdscr);
+}
+
+static void update_mini_status(void)
+{
+	wattron(stdscr, BLACKONWHITE);
+	mvwprintw(stdscr, LINES-1, COLS-31, conf.paused ? "|=" : "|>");
+	if (!conf.filter_off && (conf.do_macfilter || conf.filter_pkt != PKT_TYPE_ALL || conf.filter_mode != WLAN_MODE_ALL))
+		mvwprintw(stdscr, LINES-1, COLS-29, "|F");
+	else
+		mvwprintw(stdscr, LINES-1, COLS-29, "| ");
+	mvwprintw(stdscr, LINES-1, COLS-27, "|Ch%03d@%s",
+		channel_get_chan(conf.channel_idx),
+		channel_width_string_short(conf.channel_width, conf.channel_ht40plus));
+
+	wattroff(stdscr, BLACKONWHITE);
+	wnoutrefresh(stdscr);
+}
+
+static void update_menu(void)
+{
+	wattron(stdscr, BLACKONWHITE);
+	mvwhline(stdscr, LINES-1, 0, ' ', COLS);
+
+#define KEYMARK A_UNDERLINE
+	attron(KEYMARK); printw("Q"); attroff(KEYMARK); printw("uit ");
+	attron(KEYMARK); printw("P"); attroff(KEYMARK); printw("ause ");
+	attron(KEYMARK); printw("R"); attroff(KEYMARK); printw("eset ");
+	attron(KEYMARK); printw("H"); attroff(KEYMARK); printw("ist ");
+	attron(KEYMARK); printw("E"); attroff(KEYMARK); printw("SSID St");
+	attron(KEYMARK); printw("a"); attroff(KEYMARK); printw("ts ");
+	attron(KEYMARK); printw("S"); attroff(KEYMARK); printw("pec ");
+	attron(KEYMARK); printw("F"); attroff(KEYMARK); printw("ilt ");
+	attron(KEYMARK); printw("C"); attroff(KEYMARK); printw("han ");
+	attron(KEYMARK); printw("?"); attroff(KEYMARK); printw(" ");
+	if (show_win == NULL) {
+		printw("s"); attron(KEYMARK); printw("O"); attroff(KEYMARK); printw("rt");
+	}
+	if (show_win != NULL && show_win_current == 's') {
+		attron(KEYMARK); printw("N"); attroff(KEYMARK); printw("odes");
+	}
+#undef KEYMARK
+	mvwprintw(stdscr, LINES-1, COLS-17, "|%7s",
+		  conf.serveraddr[0] != '\0' ? conf.serveraddr : conf.ifname);
+	wattroff(stdscr, BLACKONWHITE);
+
+	update_mini_status();
+	update_clock(&time_real.tv_sec);
+}
+
+/******************* WINDOW MANAGEMENT / UPDATE *******************/
+
+static void update_show_win(void)
+{
+	if (show_win_current == 'e')
+		update_essid_win(show_win);
+	else if (show_win_current == 'h')
+		update_history_win(show_win);
+	else if (show_win_current == 'a')
+		update_statistics_win(show_win);
+	else if (show_win_current == 's')
+		update_spectrum_win(show_win);
+	else if (show_win_current == '?')
+		update_help_win(show_win);
+}
+
+static void show_window(int which)
+{
+	if (show_win != NULL && show_win_current == which) {
+		delwin(show_win);
+		show_win = NULL;
+		show_win_current = 0;
+		update_menu();
+		return;
+	}
+	if (show_win == NULL) {
+		show_win = newwin(LINES-1, COLS, 0, 0);
+		scrollok(show_win, FALSE);
+	}
+	show_win_current = which;
+	update_show_win();
+	update_menu();
+}
+
+static void show_conf_window(int key)
+{
+	if (conf_win != NULL &&
+	    (conf_win_current == key || key == '\r' || key == KEY_ENTER || key == 27)) {
+		delwin(conf_win);
+		conf_win = NULL;
+		conf_win_current = 0;
+		return;
+	}
+	if (conf_win == NULL) {
+		if (key == 'f') {
+			conf_win = newwin(FILTER_WIN_HEIGHT, FILTER_WIN_WIDTH,
+					  LINES/2-FILTER_WIN_HEIGHT/2,
+					  COLS/2-FILTER_WIN_WIDTH/2);
+			update_filter_win(conf_win);
+		}
+		else if (key == 'c') {
+			conf_win = newwin(CHANNEL_WIN_HEIGHT, CHANNEL_WIN_WIDTH,
+					LINES/2-CHANNEL_WIN_HEIGHT/2,
+					COLS/2-CHANNEL_WIN_WIDTH/2);
+			update_channel_win(conf_win);
+		}
+		scrollok(conf_win, FALSE);
+		conf_win_current = key;
+	}
+}
+
+void update_display_clock(void)
+{
+	/* helper to update just the clock every second */
+	if (time_mono.tv_sec > time_mono.tv_sec) {
+		update_clock(&time_real.tv_sec);
+		doupdate();
+	}
+}
+
+void display_log(const char *string)
+{
+	print_dump_win(string, show_win == NULL);
+}
+
+void update_display(struct packet_info* pkt)
+{
+	/*
+	 * update only in specific intervals to save CPU time
+	 * if pkt is NULL we want to force an update
+	 */
+	if (pkt != NULL &&
+	    time_mono.tv_sec == last_time.tv_sec &&
+	    (time_mono.tv_nsec - last_time.tv_nsec) / 1000 < conf.display_interval ) {
+		/* just add the line to dump win so we don't loose it */
+		update_dump_win(pkt);
+		return;
+	}
+
+	if (display_resize_needed == 1) {
+		resize_display_all();
+		display_resize_needed = 0;
+	}
+
+	update_menu();
+
+	/* update clock every second */
+	if (time_mono.tv_sec > time_mono.tv_sec)
+		update_clock(&time_real.tv_sec);
+
+	last_time = time_mono;
+
+	if (show_win != NULL)
+		update_show_win();
+	else
+		update_main_win(pkt);
+
+	if (conf_win != NULL) {
+		redrawwin(conf_win);
+		wnoutrefresh(conf_win);
+	}
+
+	/* only one redraw */
+	doupdate();
+}
 
 /******************* INPUT *******************/
 
-void
-handle_user_input(void)
+void handle_user_input(void)
 {
 	int key;
 
@@ -445,16 +395,7 @@ handle_user_input(void)
 		exit(0);
 
 	case 'r': case 'R':
-		clear_display_main();
-		print_dump_win("\n- RESET -", show_win == NULL);
-		free_lists();
-		essids.split_active = 0;
-		essids.split_essid = NULL;
-		memset(&hist, 0, sizeof(hist));
-		memset(&stats, 0, sizeof(stats));
-		memset(&spectrum, 0, sizeof(spectrum));
-		init_spectrum();
-		gettimeofday(&stats.stats_time, NULL);
+		main_reset();
 		break;
 
 	/* big windows */
@@ -470,6 +411,7 @@ handle_user_input(void)
 	case 'f': case 'F':
 	case 'c': case 'C':
 	case '\r': case KEY_ENTER: /* used to close win */
+	case 27 : /* ESC */
 		show_conf_window(tolower(key));
 		break;
 	}
@@ -477,20 +419,20 @@ handle_user_input(void)
 	update_display(NULL);
 }
 
-
 /******************* INIT *******************/
 
-void
-init_display(void)
+void init_display(void)
 {
 	initscr();
 	start_color();	/* Start the color functionality */
+	use_default_colors();
 	keypad(stdscr, TRUE);
 	nonl();		/* tell curses not to do NL->CR/NL on output */
 	cbreak();	/* take input chars one at a time, no wait for \n */
 	curs_set(0);	/* don't show cursor */
 	noecho();
 	nodelay(stdscr, TRUE);
+	ESCDELAY = 25;	/* we don't use ESC sequences */
 
 	init_pair(1, COLOR_WHITE, COLOR_BLACK);
 	init_pair(2, COLOR_GREEN, COLOR_BLACK);
@@ -515,8 +457,8 @@ init_display(void)
 
 	init_display_main();
 
-	if (conf.do_change_channel)
-		show_window('s');
+	if (conf.display_view != 0)
+		show_window(conf.display_view);
 
 	update_menu();
 	update_display(NULL);
@@ -525,9 +467,12 @@ init_display(void)
 	conf.display_initialized = 1;
 }
 
-
-void
-finish_display(void)
+void finish_display(void)
 {
 	endwin();
 }
+
+void display_clear(void)
+{
+	clear_display_main();
+}
