@@ -1,6 +1,6 @@
 /* horst - Highly Optimized Radio Scanning Tool
  *
- * Copyright (C) 2005-2014 Bruno Randolf (br1@einfach.org)
+ * Copyright (C) 2005-2016 Bruno Randolf (br1@einfach.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,8 +22,7 @@
 
 struct list_head;
 
-void
-listsort(struct list_node *list,
-	int(*cmp)(const struct list_node*, const struct list_node*));
+void listsort(struct list_node *list,
+	      int(*cmp)(const struct list_node*, const struct list_node*));
 
 #endif
