@@ -1,6 +1,6 @@
 /* horst - Highly Optimized Radio Scanning Tool
  *
- * Copyright (C) 2005-2014 Bruno Randolf (br1@einfach.org)
+ * Copyright (C) 2005-2016 Bruno Randolf (br1@einfach.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,12 +21,13 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <getopt.h>
 #include <signal.h>
 #include <sys/time.h>
+#include <time.h>
 #include <errno.h>
 #include <err.h>
 #include <sys/socket.h>
+#include <net/if.h>
 
 #include "main.h"
 #include "util.h"
@@ -40,27 +41,24 @@
 #include "channel.h"
 #include "node.h"
 #include "essid.h"
+#include "conf_options.h"
+#include "ifctrl.h"
 
+#ifdef __MACH__
+#include "mach_clock_gettime.h"
+#endif
 
 struct list_head nodes;
 struct essid_meta_info essids;
 struct history hist;
 struct statistics stats;
 struct channel_info spectrum[MAX_CHANNELS];
+struct node_names_info node_names;
 
-struct config conf = {
-	.node_timeout		= NODE_TIMEOUT,
-	.channel_time		= CHANNEL_TIME,
-	.ifname			= INTERFACE_NAME,
-	.display_interval	= DISPLAY_UPDATE_INTERVAL,
-	.recv_buffer_size	= RECV_BUFFER_SIZE,
-	.port			= DEFAULT_PORT,
-	.control_pipe		= DEFAULT_CONTROL_PIPE,
-	.filter_pkt		= PKT_TYPE_ALL,
-	.filter_mode		= WLAN_MODE_ALL,
-};
+struct config conf;
 
-struct timeval the_time;
+struct timespec time_mono;
+struct timespec time_real;
 
 int mon; /* monitoring socket */
 
@@ -92,8 +90,8 @@ static size_t cli_buflen;
 static fd_set read_fds;
 static fd_set write_fds;
 static fd_set excpt_fds;
-static struct timeval tv;
 
+static volatile sig_atomic_t is_sigint_caught;
 
 void __attribute__ ((format (printf, 1, 2)))
 printlog(const char *fmt, ...)
@@ -106,7 +104,7 @@ printlog(const char *fmt, ...)
 	va_end(ap);
 
 	if (conf.quiet || conf.debug || !conf.display_initialized)
-		printf("%s\n", &buf[1]);
+		fprintf(stderr, "%s\n", &buf[1]);
 	else {
 		/* fix up string for display log */
 		buf[0] = '\n';
@@ -114,9 +112,7 @@ printlog(const char *fmt, ...)
 	}
 }
 
-
-static void
-update_history(struct packet_info* p)
+static void update_history(struct packet_info* p)
 {
 	if (p->phy_signal == 0)
 		return;
@@ -131,9 +127,7 @@ update_history(struct packet_info* p)
 		hist.index = 0;
 }
 
-
-static void
-update_statistics(struct packet_info* p)
+static void update_statistics(struct packet_info* p)
 {
 	int type = (p->phy_flags & PHY_FLAG_BADFCS) ? 1 : p->wlan_type;
 
@@ -160,9 +154,7 @@ update_statistics(struct packet_info* p)
 	}
 }
 
-
-static void
-update_spectrum(struct packet_info* p, struct node_info* n)
+static void update_spectrum(struct packet_info* p, struct node_info* n)
 {
 	struct channel_info* chan;
 	struct chan_node* cn;
@@ -206,9 +198,7 @@ update_spectrum(struct packet_info* p, struct node_info* n)
 	cn->packets++;
 }
 
-
-void
-update_spectrum_durations(void)
+void update_spectrum_durations(void)
 {
 	/* also if channel was not changed, keep stats only for every channel_time.
 	 * display code uses durations_last to get a more stable view */
@@ -221,120 +211,146 @@ update_spectrum_durations(void)
 	}
 }
 
-
-static void 
-write_to_file(struct packet_info* p)
+static void write_to_file(struct packet_info* p)
 {
+	char buf[40];
+	int i;
+	struct tm* ltm = localtime(&time_real.tv_sec);
+
+	//timestamp, e.g. 2015-05-16 15:05:44.338806 +0300
+	i = strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", ltm);
+	i += snprintf(buf + i, sizeof(buf) - i, ".%06ld", (long)(time_mono.tv_nsec / 1000));
+	i += strftime(buf + i, sizeof(buf) - i, " %z", ltm);
+	fprintf(DF, "%s, ", buf);
+
 	fprintf(DF, "%s, %s, ",
 		get_packet_type_name(p->wlan_type), ether_sprintf(p->wlan_src));
 	fprintf(DF, "%s, ", ether_sprintf(p->wlan_dst));
 	fprintf(DF, "%s, ", ether_sprintf(p->wlan_bssid));
-	fprintf(DF, "%x, %d, %d, %d, %d, %d, %d, ",
-		p->pkt_types, p->phy_signal, 0, 0,
-		p->wlan_len, p->phy_rate, p->phy_freq);
+	fprintf(DF, "%x, %d, %d, %d, %d, ",
+		p->pkt_types, p->phy_signal, p->wlan_len, p->phy_rate, p->phy_freq);
 	fprintf(DF, "%016llx, ", (unsigned long long)p->wlan_tsf);
 	fprintf(DF, "%s, %d, %d, %d, %d, %d, ",
 		p->wlan_essid, p->wlan_mode, p->wlan_channel,
 		p->wlan_wep, p->wlan_wpa, p->wlan_rsn);
-	fprintf(DF, "%s, ", ip_sprintf(p->ip_src));
-	fprintf(DF, "%s, ", ip_sprintf(p->ip_dst));
-	fprintf(DF, "%d, %d\n", p->olsr_type, p->olsr_neigh);
+	fprintf(DF, "%s, %s\n", ip_sprintf(p->ip_src), ip_sprintf(p->ip_dst));
 	fflush(DF);
 }
 
-
-/* return 1 if packet is filtered */
-static int
-filter_packet(struct packet_info* p)
+/* return true if packet is filtered */
+static bool filter_packet(struct packet_info* p)
 {
 	int i;
 
 	if (conf.filter_off)
-		return 0;
+		return false;
 
+	/* if packets with bad FCS are not filtered, still we can not trust any
+	 * other header, so in any case return */
+	if (p->phy_flags & PHY_FLAG_BADFCS) {
+		if (!conf.filter_badfcs) {
+			stats.filtered_packets++;
+			return true;
+		}
+		return false;
+	}
+
+	/* filter by WLAN frame type and also type 3 which is not defined */
+	i = WLAN_FRAME_TYPE(p->wlan_type);
+	if (i == 3 || !(conf.filter_stype[i] & BIT(WLAN_FRAME_STYPE(p->wlan_type)))) {
+		stats.filtered_packets++;
+		return true;
+	}
+
+	/* filter by MODE (AP, IBSS, ...) this also filters packets where we
+	 * cannot associate a mode (ACK, RTS/CTS) */
+	if (conf.filter_mode != WLAN_MODE_ALL && ((p->wlan_mode & ~conf.filter_mode) || p->wlan_mode == 0)) {
+		stats.filtered_packets++;
+		return true;
+	}
+
+	/* filter higher level packet types */
 	if (conf.filter_pkt != PKT_TYPE_ALL && (p->pkt_types & ~conf.filter_pkt)) {
 		stats.filtered_packets++;
-		return 1;
-	}
-
-	/* cannot trust anything if FCS is bad */
-	if (p->phy_flags & PHY_FLAG_BADFCS)
-		return 0;
-
-	if (conf.filter_mode != WLAN_MODE_ALL && ((p->wlan_mode & ~conf.filter_mode) || p->wlan_mode == 0)) {
-		/* this also filters out packets where we cannot associate a mode (ACK, RTS/CTS) */
-		stats.filtered_packets++;
-		return 1;
+		return true;
 	}
 
+	/* filter BSSID */
 	if (MAC_NOT_EMPTY(conf.filterbssid) &&
 	    memcmp(p->wlan_bssid, conf.filterbssid, MAC_LEN) != 0) {
 		stats.filtered_packets++;
-		return 1;
+		return true;
 	}
 
+	/* filter MAC adresses */
 	if (conf.do_macfilter) {
 		for (i = 0; i < MAX_FILTERMAC; i++) {
 			if (MAC_NOT_EMPTY(p->wlan_src) &&
 			    conf.filtermac_enabled[i] &&
 			    memcmp(p->wlan_src, conf.filtermac[i], MAC_LEN) == 0) {
-				return 0;
+				return false;
 			}
 		}
 		stats.filtered_packets++;
-		return 1;
+		return true;
 	}
-	return 0;
+
+	return false;
 }
 
-
-void
-handle_packet(struct packet_info* p)
+static void fixup_packet_channel(struct packet_info* p)
 {
-	struct node_info* n = NULL;
 	int i = -1;
 
-	/* filter on server side only */
-	if (!conf.serveraddr && filter_packet(p)) {
-		if (!conf.quiet && !conf.paused && !conf.debug)
-			update_display_clock();
-		return;
-	}
-
-	if (cli_fd != -1)
-		net_send_packet(p);
-
-	if (conf.dumpfile != NULL && !conf.paused && DF != NULL)
-		write_to_file(p);
-
-	if (conf.paused)
-		return;
-
-	DEBUG("handle %s\n", get_packet_type_name(p->wlan_type));
-
 	/* get channel index for packet */
 	if (p->phy_freq) {
 		i = channel_find_index_from_freq(p->phy_freq);
 	}
-	/* not found from pkt, best guess from config but it might be
+
+	/* if not found from pkt, best guess from config but it might be
 	 * unknown (-1) too */
 	if (i < 0)
 		p->pkt_chan_idx = conf.channel_idx;
 	else
 		p->pkt_chan_idx = i;
+
 	/* wlan_channel is only known for beacons and probe response,
 	 * otherwise we set it from the physical channel */
 	if (p->wlan_channel == 0 && p->pkt_chan_idx >= 0)
-		p->wlan_channel = channel_get_chan_from_idx(p->pkt_chan_idx);
+		p->wlan_channel = channel_get_chan(p->pkt_chan_idx);
 
 	/* if current channel is unknown (this is a mac80211 bug), guess it from
 	 * the packet */
 	if (conf.channel_idx < 0 && p->pkt_chan_idx >= 0)
 		conf.channel_idx = p->pkt_chan_idx;
+}
 
+void handle_packet(struct packet_info* p)
+{
+	struct node_info* n = NULL;
+
+	/* filter on server side only */
+	if (conf.serveraddr[0] == '\0' && filter_packet(p)) {
+		if (!conf.quiet && !conf.paused && !conf.debug)
+			update_display_clock();
+		return;
+	}
+
+	fixup_packet_channel(p);
+
+	if (cli_fd != -1)
+		net_send_packet(p);
+
+	if (conf.dumpfile[0] != '\0' && !conf.paused && DF != NULL)
+		write_to_file(p);
+
+	if (conf.paused)
+		return;
+
+	/* we can't trust any fields except phy_* of packets with bad FCS */
 	if (!(p->phy_flags & PHY_FLAG_BADFCS)) {
-		/* we can't trust any fields except phy_* of packets with bad FCS,
-		 * so we can't do all this here */
+		DEBUG("handle %s\n", get_packet_type_name(p->wlan_type));
+
 		n = node_update(p);
 
 		if (n)
@@ -358,9 +374,7 @@ handle_packet(struct packet_info* p)
 		update_display(p);
 }
 
-
-static void
-local_receive_packet(int fd, unsigned char* buffer, size_t bufsize)
+static void local_receive_packet(int fd, unsigned char* buffer, size_t bufsize)
 {
 	int len;
 	struct packet_info p;
@@ -385,11 +399,11 @@ local_receive_packet(int fd, unsigned char* buffer, size_t bufsize)
 	handle_packet(&p);
 }
 
-
-static void
-receive_any(void)
+static void receive_any(const sigset_t *const waitmask)
 {
 	int ret, mfd;
+	uint32_t usecs = UINT32_MAX;
+	struct timespec ts;
 
 	FD_ZERO(&read_fds);
 	FD_ZERO(&write_fds);
@@ -404,13 +418,14 @@ receive_any(void)
 	if (ctlpipe != -1)
 		FD_SET(ctlpipe, &read_fds);
 
-	tv.tv_sec = 0;
-	tv.tv_usec = min(conf.channel_time, 1000000);
+	usecs = min(channel_get_remaining_dwell_time(), 1000000);
+	ts.tv_sec = usecs / 1000000;
+	ts.tv_nsec = usecs % 1000000 * 1000;
 	mfd = max(mon, srv_fd);
 	mfd = max(mfd, ctlpipe);
 	mfd = max(mfd, cli_fd) + 1;
 
-	ret = select(mfd, &read_fds, &write_fds, &excpt_fds, &tv);
+	ret = pselect(mfd, &read_fds, &write_fds, &excpt_fds, &ts, waitmask);
 	if (ret == -1 && errno == EINTR) /* interrupted */
 		return;
 	if (ret == 0) { /* timeout */
@@ -427,7 +442,7 @@ receive_any(void)
 
 	/* local packet or client */
 	if (FD_ISSET(mon, &read_fds)) {
-		if (conf.serveraddr)
+		if (conf.serveraddr[0] != '\0')
 			net_receive(mon, buffer, &buflen, sizeof(buffer));
 		else
 			local_receive_packet(mon, buffer, sizeof(buffer));
@@ -446,9 +461,7 @@ receive_any(void)
 		control_receive_command();
 }
 
-
-void
-free_lists(void)
+void free_lists(void)
 {
 	int i;
 	struct essid_info *e, *f;
@@ -470,7 +483,7 @@ free_lists(void)
 	}
 
 	/* free channel nodes */
-	for (i = 0; i < conf.num_channels; i++) {
+	for (i = 0; i < channel_get_num_channels(); i++) {
 		list_for_each_safe(&spectrum[i].nodes, cn, cn2, chan_list) {
 			DEBUG("free chan_node %p\n", cn);
 			list_del(&cn->chan_list);
@@ -480,14 +493,18 @@ free_lists(void)
 	}
 }
 
-
-static void
-finish_all(void)
+static void exit_handler(void)
 {
 	free_lists();
 
-	if (!conf.serveraddr)
-		close_packet_socket(mon, conf.ifname);
+	if (conf.serveraddr[0] == '\0') {
+		close_packet_socket(mon);
+	}
+
+	ifctrl_flags(conf.ifname, false, false);
+
+	if (conf.monitor_added)
+		ifctrl_iwdel(conf.ifname);
 
 	if (DF != NULL) {
 		fclose(DF);
@@ -502,305 +519,255 @@ finish_all(void)
 
 	if (!conf.quiet && !conf.debug)
 		finish_display();
+
+	ifctrl_finish();
 }
 
-
-static void
-exit_handler(void)
+static void sigint_handler(__attribute__((unused)) int sig)
 {
-	finish_all();
+	/* Only set an atomic flag here to keep processing in the interrupt
+	 * context as minimal as possible (at least all unsafe functions are
+	 * prohibited, see signal(7)). The flag is handled in the mainloop. */
+	is_sigint_caught = 1;
 }
 
-
-static void
-sigint_handler(__attribute__((unused)) int sig)
-{
-	exit(0);
-}
-
-
-static void
-sigpipe_handler(__attribute__((unused)) int sig)
+static void sigpipe_handler(__attribute__((unused)) int sig)
 {
 	/* ignore signal here - we will handle it after write failed */
 }
 
-
-static void
-get_options(int argc, char** argv)
-{
-	int c;
-	static int n;
-
-	while((c = getopt(argc, argv, "hqDsCi:t:c:p:e:f:d:o:b:X::x:m:u:a:")) > 0) {
-		switch (c) {
-		case 'p':
-			conf.port = optarg;
-			break;
-		case 'q':
-			conf.quiet = 1;
-			break;
-		case 'D':
-#if DO_DEBUG
-			conf.debug = 1;
-#else
-			printf("Please compile with DEBUG=1 to use the -D option!\n");
-			exit(1);
-#endif
-			break;
-
-		case 'i':
-			conf.ifname = optarg;
-			break;
-		case 'o':
-			conf.dumpfile = optarg;
-			break;
-		case 't':
-			conf.node_timeout = atoi(optarg);
-			break;
-		case 'b':
-			conf.recv_buffer_size = atoi(optarg);
-			break;
-		case 's':
-			conf.do_change_channel = 1;
-			break;
-		case 'd':
-			conf.display_interval = atoi(optarg) * 1000;
-			break;
-		case 'e':
-			if (n >= MAX_FILTERMAC)
-				break;
-			conf.do_macfilter = 1;
-			convert_string_to_mac(optarg, conf.filtermac[n]);
-			conf.filtermac_enabled[n] = 1;
-			n++;
-			break;
-		case 'c':
-			conf.serveraddr = optarg;
-			break;
-		case 'C':
-			conf.allow_client = 1;
-			break;
-		case 'u':
-			conf.channel_max = atoi(optarg);
-			break;
-		case 'X':
-			if (optarg != NULL)
-				conf.control_pipe = optarg;
-			conf.allow_control = 1;
-			break;
-		case 'x':
-			control_send_command(optarg);
-			exit(0);
-		case 'm':
-			if (conf.filter_mode == WLAN_MODE_ALL)
-				conf.filter_mode = 0;
-			if (strcmp(optarg, "AP") == 0)
-				conf.filter_mode |= WLAN_MODE_AP;
-			else if (strcmp(optarg, "STA") == 0)
-				conf.filter_mode |= WLAN_MODE_STA;
-			else if (strcmp(optarg, "ADH") == 0 || strcmp(optarg, "IBSS") == 0)
-				conf.filter_mode |= WLAN_MODE_IBSS;
-			else if (strcmp(optarg, "PRB") == 0)
-				conf.filter_mode |= WLAN_MODE_PROBE;
-			else if (strcmp(optarg, "WDS") == 0)
-				conf.filter_mode |= WLAN_MODE_4ADDR;
-			else if (strcmp(optarg, "UNKNOWN") == 0)
-				conf.filter_mode |= WLAN_MODE_UNKNOWN;
-			break;
-		case 'f':
-			if (conf.filter_pkt == PKT_TYPE_ALL)
-				conf.filter_pkt = 0;
-			if (strcmp(optarg, "CTRL") == 0 || strcmp(optarg, "CONTROL") == 0)
-				conf.filter_pkt |= PKT_TYPE_CTRL | PKT_TYPE_ALL_CTRL;
-			else if (strcmp(optarg, "MGMT") == 0 || strcmp(optarg, "MANAGEMENT") == 0)
-				conf.filter_pkt |= PKT_TYPE_MGMT | PKT_TYPE_ALL_MGMT;
-			else if (strcmp(optarg, "DATA") == 0)
-				conf.filter_pkt |= PKT_TYPE_DATA | PKT_TYPE_ALL_DATA;
-			else if (strcmp(optarg, "BADFCS") == 0)
-				conf.filter_pkt |= PKT_TYPE_BADFCS;
-			else if (strcmp(optarg, "BEACON") == 0)
-				conf.filter_pkt |= PKT_TYPE_BEACON;
-			else if (strcmp(optarg, "PROBE") == 0)
-				conf.filter_pkt |= PKT_TYPE_PROBE;
-			else if (strcmp(optarg, "ASSOC") == 0)
-				conf.filter_pkt |= PKT_TYPE_ASSOC;
-			else if (strcmp(optarg, "AUTH") == 0)
-				conf.filter_pkt |= PKT_TYPE_AUTH;
-			else if (strcmp(optarg, "RTS") == 0)
-				conf.filter_pkt |= PKT_TYPE_RTSCTS;
-			else if (strcmp(optarg, "ACK") == 0)
-				conf.filter_pkt |= PKT_TYPE_ACK;
-			else if (strcmp(optarg, "NULL") == 0)
-				conf.filter_pkt |= PKT_TYPE_NULL;
-			else if (strcmp(optarg, "QDATA") == 0)
-				conf.filter_pkt |= PKT_TYPE_QDATA;
-			else if (strcmp(optarg, "ARP") == 0)
-				conf.filter_pkt |= PKT_TYPE_ARP;
-			else if (strcmp(optarg, "IP") == 0)
-				conf.filter_pkt |= PKT_TYPE_IP;
-			else if (strcmp(optarg, "ICMP") == 0)
-				conf.filter_pkt |= PKT_TYPE_ICMP;
-			else if (strcmp(optarg, "UDP") == 0)
-				conf.filter_pkt |= PKT_TYPE_UDP;
-			else if (strcmp(optarg, "TCP") == 0)
-				conf.filter_pkt |= PKT_TYPE_TCP;
-			else if (strcmp(optarg, "OLSR") == 0)
-				conf.filter_pkt |= PKT_TYPE_OLSR;
-			else if (strcmp(optarg, "BATMAN") == 0)
-				conf.filter_pkt |= PKT_TYPE_BATMAN;
-			else if (strcmp(optarg, "MESHZ") == 0)
-				conf.filter_pkt |= PKT_TYPE_MESHZ;
-			/* if one of the individual subtype frames is selected we enable the general frame type */
-			if (conf.filter_pkt & PKT_TYPE_ALL_MGMT)
-				conf.filter_pkt |= PKT_TYPE_MGMT;
-			if (conf.filter_pkt & PKT_TYPE_ALL_CTRL)
-				conf.filter_pkt |= PKT_TYPE_CTRL;
-			if (conf.filter_pkt & PKT_TYPE_ALL_DATA)
-				conf.filter_pkt |= PKT_TYPE_DATA;
-			break;
-		case 'h':
-		default:
-			printf("\nUsage: %s [-h] [-q] [-D ] [-i interface] [-t sec] [-d ms] [-b bytes]\n"
-				"\t\t[-s] [-u] [-C] [-c IP] [-p port] [-o file] [-X[name]] [-x command]\n"
-				"\t\t[-e MAC] [-f PKT_NAME] [-m MODE]\n\n"
-
-				"General Options: Description (default value)\n"
-				"  -h\t\tHelp\n"
-				"  -q\t\tQuiet, no output\n"
-#if DO_DEBUG
-				"  -D\t\tShow lots of debug output, no UI\n"
-#endif
-				"  -i <intf>\tInterface name (wlan0)\n"
-				"  -t <sec>\tNode timeout in seconds (60)\n"
-				"  -d <ms>\tDisplay update interval in ms (100)\n"
-				"  -b <bytes>\tReceive buffer size in bytes (not set)\n"
-
-				"\nFeature Options:\n"
-				"  -s\t\t(Poor mans) Spectrum analyzer mode\n"
-				"  -u\t\tUpper channel limit\n\n"
-
-				"  -C\t\tAllow client connection, server mode (off)\n"
-				"  -c <IP>\tConnect to server with <IP>, client mode (off)\n"
-				"  -p <port>\tPort number of server (4444)\n\n"
-
-				"  -o <filename>\tWrite packet info into 'filename'\n\n"
-
-				"  -X[filename]\tAllow control socket on 'filename' (/tmp/horst)\n"
-				"  -x <command>\tSend control command\n"
-
-				"\nFilter Options:\n"
-				" Filters are generally 'positive' or 'inclusive' which means you define\n"
-				" what you want to see, and everything else is getting filtered out.\n"
-				" If a filter is not set it is inactive and nothing is filtered.\n"
-				" All filter options can be specified multiple times.\n"
-				"  -e <MAC>\tSource MAC addresses (xx:xx:xx:xx:xx:xx), up to 9 times\n"
-				"  -f <PKT_NAME>\tFilter packet types\n"
-				"  -m <MODE>\tOperating mode: AP|STA|ADH|PRB|WDS|UNKNOWN\n"
-				"\n",
-				argv[0]);
-			exit(0);
-			break;
-		}
-	}
-}
-
-
-void
-init_spectrum(void)
+void init_spectrum(void)
 {
 	int i;
 
-	for (i = 0; i < conf.num_channels && i < MAX_CHANNELS; i++) {
+	for (i = 0; i < MAX_CHANNELS; i++) {
 		list_head_init(&spectrum[i].nodes);
 		ewma_init(&spectrum[i].signal_avg, 1024, 8);
 		ewma_init(&spectrum[i].durations_avg, 1024, 8);
 	}
 }
 
-
-int
-main(int argc, char** argv)
+static void mac_name_file_read(const char* filename)
 {
+	FILE* fp;
+	char line[255];
+	char macs[18];
+	char name[18];
+	int idx = 0;
+	int n;
+
+	if ((fp = fopen(filename, "r")) == NULL) {
+		printlog("Could not open mac name file '%s'", filename);
+		return;
+	}
+
+	while (fgets(line, sizeof(line), fp) != NULL) {
+		// first try dnsmasq dhcp.leases format
+		n = sscanf(line, "%*s %17s %*s %17s", macs, name);
+		if (n < 2) // if not MAC name
+			n = sscanf(line, "%17s %17s", macs, name);
+		if (n == 2) {
+			convert_string_to_mac(macs, node_names.entry[idx].mac);
+			strncpy(node_names.entry[idx].name, name, MAX_NODE_NAME_STRLEN);
+			node_names.entry[idx].name[MAX_NODE_NAME_STRLEN] = '\0';
+			idx++;
+		}
+	}
+
+	fclose(fp);
+
+	node_names.count = idx;
+
+	for (n = 0; n < node_names.count; n++) {
+		printlog("MAC %s = %s", ether_sprintf(node_names.entry[n].mac),
+			 node_names.entry[n].name );
+	}
+}
+
+const char* mac_name_lookup(const unsigned char* mac, int shorten_mac)
+{
+	int i;
+	if (conf.mac_name_lookup) {
+		for (i = 0; i < node_names.count; i++) {
+			if (memcmp(node_names.entry[i].mac, mac, MAC_LEN) == 0)
+				return node_names.entry[i].name;
+		}
+	}
+	return shorten_mac ? ether_sprintf_short(mac) : ether_sprintf(mac);
+}
+
+static void generate_mon_ifname(char *const buf, const size_t buf_size)
+{
+	unsigned int i;
+
+	for (i=0;; ++i) {
+		int len;
+
+		len = snprintf(buf, buf_size, "horst%d", i);
+		if (len < 0)
+			err(1, "failed to generate monitor interface name");
+		if ((unsigned int) len >= buf_size)
+			errx(1, "failed to generate a sufficiently short "
+			     "monitor interface name");
+		if (!if_nametoindex(buf))
+			break;  /* interface does not exist yet, done */
+	}
+}
+
+int main(int argc, char** argv)
+{
+	sigset_t workmask;
+	sigset_t waitmask;
+	struct sigaction sigint_action;
+	struct sigaction sigpipe_action;
+
 	list_head_init(&essids.list);
 	list_head_init(&nodes);
+	init_spectrum();
 
-	get_options(argc, argv);
+	config_parse_file_and_cmdline(argc, argv);
+
+	sigint_action.sa_handler = sigint_handler;
+	sigemptyset(&sigint_action.sa_mask);
+	sigint_action.sa_flags = 0;
+	sigaction(SIGINT, &sigint_action, NULL);
+	sigaction(SIGTERM, &sigint_action, NULL);
+	sigaction(SIGHUP, &sigint_action, NULL);
+
+	sigpipe_action.sa_handler = sigpipe_handler;
+	sigaction(SIGPIPE, &sigpipe_action, NULL);
 
-	signal(SIGINT, sigint_handler);
-	signal(SIGTERM, sigint_handler);
-	signal(SIGHUP, sigint_handler);
-	signal(SIGPIPE, sigpipe_handler);
 	atexit(exit_handler);
 
-	gettimeofday(&stats.stats_time, NULL);
-	gettimeofday(&the_time, NULL);
+	clock_gettime(CLOCK_MONOTONIC, &stats.stats_time);
+	clock_gettime(CLOCK_MONOTONIC, &time_mono);
 
 	conf.channel_idx = -1;
 
+	if (conf.mac_name_lookup)
+		mac_name_file_read(conf.mac_name_file);
+
 	if (conf.allow_control) {
-		printlog("Allowing control socket");
+		printlog("Allowing control socket '%s'", conf.control_pipe);
 		control_init_pipe();
 	}
 
-	if (conf.serveraddr)
+	if (conf.serveraddr[0] != '\0')
 		mon = net_open_client_socket(conf.serveraddr, conf.port);
 	else {
+		ifctrl_init();
+		ifctrl_iwget_interface_info(conf.ifname);
+
+		/* if the interface is not already in monitor mode, try to set
+		 * it to monitor or create an additional virtual monitor interface */
+		if (conf.add_monitor || (!ifctrl_is_monitor() &&
+					 !ifctrl_iwset_monitor(conf.ifname))) {
+			char mon_ifname[IF_NAMESIZE];
+			generate_mon_ifname(mon_ifname, IF_NAMESIZE);
+			if (!ifctrl_iwadd_monitor(conf.ifname, mon_ifname))
+				err(1, "failed to add virtual monitor interface");
+
+			printlog("INFO: A virtual interface '%s' will be used "
+				 "instead of '%s'.", mon_ifname, conf.ifname);
+
+			strncpy(conf.ifname, mon_ifname, IF_NAMESIZE);
+			conf.monitor_added = 1;
+			/* Now we have a new monitor interface, proceed
+			 * normally. The interface will be deleted at exit. */
+		}
+
+		if (!ifctrl_flags(conf.ifname, true, true))
+			err(1, "failed to bring interface '%s' up",
+			    conf.ifname);
+
+		/* get info again, as chan width is only available on UP interfaces */
+		ifctrl_iwget_interface_info(conf.ifname);
+
 		mon = open_packet_socket(conf.ifname, conf.recv_buffer_size);
 		if (mon <= 0)
 			err(1, "Couldn't open packet socket");
+		conf.arphrd = device_get_hwinfo(mon, conf.ifname,
+						conf.my_mac_addr);
 
-		conf.arphrd = device_get_hwinfo(mon, conf.ifname, conf.my_mac_addr);
 		if (conf.arphrd != ARPHRD_IEEE80211_PRISM &&
-		    conf.arphrd != ARPHRD_IEEE80211_RADIOTAP) {
-			printf("You need to put your interface into monitor mode!\n");
-			printf("(e.g. 'iw %s interface add mon0 type monitor' and 'horst -i mon0')\n", conf.ifname);
-			exit(1);
-		}
+		    conf.arphrd != ARPHRD_IEEE80211_RADIOTAP)
+			err(1, "interface '%s' is not in monitor mode",
+			    conf.ifname);
 
-		channel_init();
-		init_spectrum();
+		if (!channel_init() && conf.quiet)
+			err(1, "failed to change the initial channel number");
 	}
 
+	printf("Max PHY rate: %d Mbps\n", conf.max_phy_rate/10);
+
 	if (!conf.quiet && !conf.debug)
 		init_display();
 
-	if (conf.dumpfile != NULL)
-		dumpfile_open(conf.dumpfile);
-
-	if (!conf.serveraddr && conf.port && conf.allow_client)
+	if (conf.serveraddr[0] == '\0' && conf.port && conf.allow_client)
 		net_init_server_socket(conf.port);
 
-	for ( /* ever */ ;;)
-	{
-		receive_any();
-		gettimeofday(&the_time, NULL);
-		timeout_nodes();
+	/* Race-free signal handling:
+	 *   1. block all handled signals while working (with workmask)
+	 *   2. receive signals *only* while waiting in pselect() (with waitmask)
+	 *   3. switch between these two masks atomically with pselect()
+	 */
+	if (sigemptyset(&workmask)                       == -1 ||
+	    sigaddset(&workmask, SIGINT)                 == -1 ||
+	    sigaddset(&workmask, SIGHUP)                 == -1 ||
+	    sigaddset(&workmask, SIGTERM)                == -1 ||
+	    sigprocmask(SIG_BLOCK, &workmask, &waitmask) == -1)
+		err(1, "failed to block signals: %m");
 
-		if (!conf.serveraddr) { /* server */
-			if (channel_auto_change()) {
+	while (!conf.do_change_channel || conf.channel_scan_rounds != 0)
+	{
+		receive_any(&waitmask);
+
+		if (is_sigint_caught)
+			exit(1);
+
+		clock_gettime(CLOCK_MONOTONIC, &time_mono);
+		clock_gettime(CLOCK_REALTIME, &time_real);
+
+		node_timeout();
+
+		if (conf.serveraddr[0] == '\0') { /* server */
+			if (!conf.paused && channel_auto_change()) {
 				net_send_channel_config();
 				update_spectrum_durations();
 				if (!conf.quiet && !conf.debug)
 					update_display(NULL);
+
+				if (channel_get_chan(conf.channel_idx) == conf.channel_set_num
+				    && conf.channel_scan_rounds > 0)
+					--conf.channel_scan_rounds;
 			}
 		}
 	}
-	/* will never */
 	return 0;
 }
 
-
-void
-main_pause(int pause)
+void main_pause(int pause)
 {
 	conf.paused = pause;
 	printlog(conf.paused ? "- PAUSED -" : "- RESUME -");
 }
 
+void main_reset(void)
+{
+	if (!conf.quiet && !conf.debug)
+		display_clear();
+	printlog("- RESET -");
+	free_lists();
+	essids.split_active = 0;
+	essids.split_essid = NULL;
+	memset(&hist, 0, sizeof(hist));
+	memset(&stats, 0, sizeof(stats));
+	memset(&spectrum, 0, sizeof(spectrum));
+	init_spectrum();
+	clock_gettime(CLOCK_MONOTONIC, &stats.stats_time);
+}
 
-void
-dumpfile_open(char* name)
+void dumpfile_open(const char* name)
 {
 	if (DF != NULL) {
 		fclose(DF);
@@ -809,15 +776,20 @@ dumpfile_open(char* name)
 
 	if (name == NULL || strlen(name) == 0) {
 		printlog("- Not writing outfile");
-		conf.dumpfile = NULL;
+		conf.dumpfile[0] = '\0';
 		return;
 	}
 
-	conf.dumpfile = name;
+	strncpy(conf.dumpfile, name, MAX_CONF_VALUE_STRLEN);
+	conf.dumpfile[MAX_CONF_VALUE_STRLEN] = '\0';
 	DF = fopen(conf.dumpfile, "w");
 	if (DF == NULL)
 		err(1, "Couldn't open dump file");
 
+	fprintf(DF, "TIME, WLAN TYPE, MAC SRC, MAC DST, BSSID, PACKET TYPES, SIGNAL, ");
+	fprintf(DF, "LENGTH, PHY RATE, FREQUENCY, TSF, ESSID, MODE, CHANNEL, ");
+	fprintf(DF, "WEP, WPA1, RSN (WPA2), IP SRC, IP DST\n");
+
 	printlog("- Writing to outfile %s", conf.dumpfile);
 }
 
