@@ -1,6 +1,6 @@
 /* horst - Highly Optimized Radio Scanning Tool
  *
- * Copyright (C) 2005-2014 Bruno Randolf (br1@einfach.org)
+ * Copyright (C) 2005-2016 Bruno Randolf (br1@einfach.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,9 +20,11 @@
 #ifndef _PROTOCOL_PARSER_H_
 #define _PROTOCOL_PARSER_H_
 
+#include <stdbool.h>
+#include <stddef.h>
+
 struct packet_info;
 
-int
-parse_packet(unsigned char* buf, int len, struct packet_info* p);
+bool parse_packet(unsigned char* buf, size_t len, struct packet_info* p);
 
 #endif
